part of hakkit.hakk.builder.basic;

// memory manager for the compiler
class _MMU{
  static final List<_MemUnit> memoryPool = new List<_MemUnit>();
  
  static void addMemoryRange(String name, int startRange, int endRange){
    if (memoryPool.any((m) => m.blockName == name)){
      throw 'Cannot add memory to the MMU that is already defined with name "${name}"';
    }
    
    _log.fine('Adding $name to memory pool, starting at $startRange, ending $endRange');
    for(int i = startRange; i < endRange; i++){
      memoryPool.add(new _MemUnit(name, i));    
    }
  }
  
  // deallocates memory for the given context back to the memory pool.
  static void dalloc(String context){
    _log.fine('Attempting to dalloc() $context');
    memoryPool
      .where((m) => m.context == context)
      .forEach((m){
        _log.fine('   deallocating ${m.blockName}${m.index} ${m.context}');
        m.context = '';
        m.allocated = false;
      });
  }
  
  // attempts to allocate a memory unit from the pool.
  static _HakkObject malloc([String context = '']){
    final mu = memoryPool.firstWhere((m) => !m.allocated, orElse: () => null);
    if (mu == null){
      throw 'Compiler ran out of memory.  You will need to add another Block.system()'
      ' to your code in order to compile this hakk.';
    }
       
    mu.context = context;
    mu.allocated = true;
    
    _log.fine('malloc() ${mu.context} - ${mu.reference}');
    
    return mu.reference;
  }
  
  static void clear(){
    memoryPool.clear();
  }
}