part of hakkit.hakk.builder.basic;

/// Exposes some library internals for debugging/inspection purposes.
class Debugger
{
  /// @returns number of allocated memory units.
  int get MMUAllocated => _MMU.memoryPool.where((e) => e.allocated).length;
  
  int get MMUTotal => _MMU.memoryPool.length;
  
  int get MMUAvailable => MMUTotal - MMUAllocated;
  
  int get totalBlockDeclarations => _blockDeclarations.length;
  
  int get totalExpressionQueue => _expressionQueue.length;
  
  int get totalIfStack => _ifStack.length;
  
  int get totalWhileStack => _whileStack.length;
  
  int get totalCodeBlocks => _codeBlocks.length;
  
  int get systemBlockIndex => Block._sysBlockIndex;
  
  int get totalGlobalContext => _globalContext.length;
  
  int get labelIndex => _labelId;
  
  String get currentCodeBlock => _lastCodeBlock == null ? '{undefined}' : _lastCodeBlock;
  
  void dump(){
    print('-------------');
    print('Debugger Dump');
    print('-------------');
    print('MMU:');
    print('   Total Memory Units Available: ${MMUTotal}');
    print('   MUs Allocated: ${MMUAllocated}');
    print('   MUs Available: ${MMUAvailable}');
    print('Compiler State:');
    print('   # Block Declarations: ${totalBlockDeclarations}');
    print('   # Expression Queue Nodes: ${totalExpressionQueue}');
    print('   Total Global Context: ${totalGlobalContext}');
    print('   Total IF stack: ${totalIfStack}');
    print('   Total WHILE stack: ${totalWhileStack}');
    print('   # Code blocks: ${totalCodeBlocks}');
    print('   Current code block: ${currentCodeBlock}');
    print('   System Block Index: ${systemBlockIndex}');
    print('   Label index: ${labelIndex}');
  }
}